import 'dart:convert';
import 'dart:io';

import 'package:carrier_info/carrier_info.dart';
import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:device_info/device_info.dart';
import 'package:event_analysis_library/src/shared/model/event/event_model.dart';
import 'package:event_analysis_library/src/shared/model/meta/device_info_model.dart';
import 'package:event_analysis_library/src/shared/model/meta/device_meta_model.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:geolocator/geolocator.dart';
import 'package:package_info_plus/package_info_plus.dart';

class Utils {
  getDeviceMeta(String sesId, Position? location, String locale) async {
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    var connectivityResult = await (Connectivity().checkConnectivity());
    var mediaQuery = MediaQueryData.fromWindow(WidgetsBinding.instance!.window);
    final DeviceInfoPlugin deviceInfoPlugin = new DeviceInfoPlugin();
    Map<String, dynamic> deviceData = <String, dynamic>{};

    try {
      if (Platform.isAndroid) {
        deviceData = _readAndroidBuildData(await deviceInfoPlugin.androidInfo);
      } else if (Platform.isIOS) {
        deviceData = _readIosDeviceInfo(await deviceInfoPlugin.iosInfo);
      }
    } on PlatformException {
      deviceData = <String, dynamic>{
        'Error:': 'Failed to get platform version.'
      };
    }
    String? carrierName = await CarrierInfo.carrierName;

    var os = Platform.isAndroid ? "Android" : "IOS";
    DeviceMeta deviceMeta = DeviceMeta(
      os: os,
      osVersion: os == "Android" ? deviceData["version.release"] : Platform.operatingSystemVersion,
      applicationid: "",
      versionNum: packageInfo.version,
      versionCode: packageInfo.buildNumber,
      flavor: "",
      width: mediaQuery.size.width.toString(),
      height: mediaQuery.size.height.toString(),
      density: ((mediaQuery.size.width * mediaQuery.devicePixelRatio)
              .toStringAsFixed(0) +
          "x" +
          (mediaQuery.size.height * mediaQuery.devicePixelRatio)
              .toStringAsFixed(0)),
      densityDpi: (mediaQuery.devicePixelRatio * 160).toString(),
      orientation: mediaQuery.orientation == Orientation.portrait
          ? "Portrait"
          : "Landscape",
      locale: locale,
      latitude: location == null ? "" : location.latitude.toString(),
      longitude: location == null ? "" : location.longitude.toString(),
      batteryLevel: "",
      batteryTemp: "",
      network: connectivityResult == ConnectivityResult.mobile
          ? "Data"
          : connectivityResult == ConnectivityResult.wifi
              ? "WIFI"
              : "n/a",
      connectedToNetwork: (connectivityResult == ConnectivityResult.mobile ||
              connectivityResult == ConnectivityResult.wifi)
          ? true
          : false,
      deviceId: os == "Android"
          ? deviceData["androidId"]
          : deviceData["identifierForVendor"],
      carrier: carrierName!,
      ram: "",
      internalMemTotal: "",
      internalMemAvail: "",
      sessionId: sesId,
      brand: os == "Android" ? deviceData["brand"] : "Apple",
      device: deviceData["model"],
      deviceType: mediaQuery.size.shortestSide < 600 ? "Mobile" : "Tablet",
    );

    return json.encode(DeviceInfo(deviceMeta).toJson());
  }

  Map<String, dynamic> _readAndroidBuildData(AndroidDeviceInfo build) {
    return <String, dynamic>{
      'version.securityPatch': build.version.securityPatch,
      'version.sdkInt': build.version.sdkInt,
      'version.release': build.version.release,
      'version.previewSdkInt': build.version.previewSdkInt,
      'version.incremental': build.version.incremental,
      'version.codename': build.version.codename,
      'version.baseOS': build.version.baseOS,
      'board': build.board,
      'bootloader': build.bootloader,
      'brand': build.brand,
      'device': build.device,
      'display': build.display,
      'fingerprint': build.fingerprint,
      'hardware': build.hardware,
      'host': build.host,
      'id': build.id,
      'manufacturer': build.manufacturer,
      'model': build.model,
      'product': build.product,
      'supported32BitAbis': build.supported32BitAbis,
      'supported64BitAbis': build.supported64BitAbis,
      'supportedAbis': build.supportedAbis,
      'tags': build.tags,
      'type': build.type,
      'isPhysicalDevice': build.isPhysicalDevice,
      'androidId': build.androidId,
      'systemFeatures': build.systemFeatures,
    };
  }

  Map<String, dynamic> _readIosDeviceInfo(IosDeviceInfo data) {
    return <String, dynamic>{
      'name': data.name,
      'systemName': data.systemName,
      'systemVersion': data.systemVersion,
      'model': data.model,
      'localizedModel': data.localizedModel,
      'identifierForVendor': data.identifierForVendor,
      'isPhysicalDevice': data.isPhysicalDevice,
      'utsname.sysname:': data.utsname.sysname,
      'utsname.nodename:': data.utsname.nodename,
      'utsname.release:': data.utsname.release,
      'utsname.version:': data.utsname.version,
      'utsname.machine:': data.utsname.machine,
    };
  }

  getEventData(String eventName, String eventValue, String userId,
      String sessionId, Offset position) {
    Event event = Event(eventName, eventValue, userId, sessionId,
        position.dx.toString(), position.dy.toString());
    return json.encode(event.toJson());
  }

  getSessionId() {
    return DateTime.now().millisecondsSinceEpoch.toString();
  }

  isLocationEnabled() async {
    bool enabled = await Geolocator.isLocationServiceEnabled();
    return enabled;
  }

  createCredential(String username, String password) {
    Codec<String, String> stringToBase64 = latin1.fuse(base64);
    String credentials = "$username:$password";
    String encoded = stringToBase64.encode(credentials);
    return "Basic $encoded";
  }
}
